#
#
#
function(enable_fkb_firmware target)
  get_target_property(binary_dir ${target} BINARY_DIR)

  set(elf_target ${target}.elf)
  set(bin_target ${target}.bin)
  set(fkb_elf_target ${target}-fkb.elf)
  set(fkb_bin_target ${target}-fkb.bin)

  set(elf_file ${binary_dir}/${target}.elf)
  set(bin_file ${binary_dir}/${target}.bin)
  set(fkb_elf_file ${binary_dir}/${target}-fkb.elf)
  set(fkb_bin_file ${binary_dir}/${target}-fkb.bin)

  add_custom_command(
    OUTPUT ${fkb_elf_file}
    DEPENDS ${elf_file}
    COMMAND ${loading_PATH}/tools/mkfirmware.py --elf ${elf_file} --fkb ${fkb_elf_file}
    WORKING_DIRECTORY ${binary_dir}
    )

  add_custom_target(${fkb_elf_target} ALL DEPENDS ${fkb_elf_file})
  add_dependencies(${fkb_elf_target} ${elf_target})

  set_target_properties(${fkb_elf_target} PROPERTIES OUTPUT_NAME ${fkb_elf_file})

  add_custom_command(
    OUTPUT ${fkb_bin_file}
    DEPENDS ${fkb_elf_file}
    COMMAND ${ARDUINO_OBJCOPY} -O binary ${fkb_elf_file} ${fkb_bin_file}
    WORKING_DIRECTORY ${binary_dir}
    )

  add_custom_target(${fkb_bin_target} ALL DEPENDS ${fkb_bin_file})
  add_dependencies(${fkb_bin_target} ${fkb_elf_target})

  set_target_properties(${fkb_bin_target} PROPERTIES OUTPUT_NAME ${fkb_bin_file})

  set_property(DIRECTORY APPEND PROPERTY ADDITIONAL_MAKE_CLEAN_FILES
    "${CMAKE_CURRENT_BINARY_DIR}/${target}.syms"
    "${CMAKE_CURRENT_BINARY_DIR}/${target}.s"
    "${CMAKE_CURRENT_BINARY_DIR}/${target}.elf"
    "${CMAKE_CURRENT_BINARY_DIR}/${target}.bin"
    "${CMAKE_CURRENT_BINARY_DIR}/${target}.fkb"
    "${CMAKE_CURRENT_BINARY_DIR}/${target}-fkb.bin"
    "${CMAKE_CURRENT_BINARY_DIR}/${target}.map")
endfunction()

function(enable_samd09_firmware target)
endfunction()

if (NOT DEFINED ARDUINO_IDE)
  foreach(path $ENV{HOME}/arduino-1.8.3 $ENV{HOME}/conservify/arduino-1.8.3
               $ENV{HOME}/workspace/arduino-1.8.3
               ${PROJECT_SOURCE_DIR}/../arduino-1.8.3 ${PROJECT_SOURCE_DIR}/../../arduino-1.8.3)
      if (EXISTS ${path})
        set(ARDUINO_IDE ${path})
        break()
    endif()
  endforeach()

  if (NOT DEFINED ARDUINO_IDE)
    message(FATAL_ERROR "Unable to find Arduino IDE")
  endif()
endif()
